/*
 * Copyright 2022-2023 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package xyz.zhouxy.plusone.commons.util;

import java.util.Arrays;
import java.util.function.Supplier;

import com.google.common.base.Preconditions;

/**
 * Guava Preconditions 的扩展。
 *
 * @author ZhouXY
 * 
 * @see com.google.common.base.Preconditions
 */
public class PreconditionsExt {

    public static <E extends Throwable> void check(boolean condition, Supplier<E> e) throws E {
        if (!condition) {
            throw e.get();
        }
    }

    public static <T, E extends Throwable> void checkAllNotNull(Iterable<T> values) throws E {
        Preconditions.checkNotNull(values);
        values.forEach(Preconditions::checkNotNull);
    }

    @SafeVarargs
    public static <T, E extends Throwable> void checkAllNotNull(T... values) throws E {
        Preconditions.checkNotNull(values);
        Arrays.stream(values).forEach(Preconditions::checkNotNull);
    }

    private PreconditionsExt() {
        throw new IllegalStateException("Utility class");
    }
}
